@@ -31,7 +31,7 @@
 struct _CouchdbDocument {
 	GObject		parent;
 
-	CouchdbSession  *couchdb;
+	CouchdbSession  *session;
 	char		*dbname;
 	JsonNode	*root_node;
 };
@@ -60,7 +60,7 @@ couchdb_document_class_init (CouchdbDocumentClass *klass)
 static void
 couchdb_document_init (CouchdbDocument *document)
 {
-	document->couchdb = NULL;
+	document->session = NULL;
 	document->root_node = NULL;
 	document->dbname = NULL;
 }
@@ -75,12 +75,12 @@ couchdb_document_init (CouchdbDocument *document)
  * Return value: A newly-created #CouchdbDocument object, with no data on it.
  */
 CouchdbDocument *
-couchdb_document_new (CouchdbSession *couchdb)
+couchdb_document_new (CouchdbSession *session)
 {
 	CouchdbDocument *document;
 
 	document = g_object_new (COUCHDB_TYPE_DOCUMENT, NULL);
-	document->couchdb = couchdb;
+	document->session = session;
 	document->dbname = NULL;
 
 	document->root_node = json_node_new (JSON_NODE_OBJECT);
@@ -89,6 +89,21 @@ couchdb_document_new (CouchdbSession *couchdb)
 	return document;
 }
 
+CouchdbDocument *
+couchdb_document_new_from_json_object (CouchdbSession *session, JsonObject *json_object)
+{
+	CouchdbDocument *document;
+
+	document = g_object_new (COUCHDB_TYPE_DOCUMENT, NULL);
+	document->session = session;
+	document->dbname = NULL;
+
+	document->root_node = json_node_new (JSON_NODE_OBJECT);
+	json_node_set_object (document->root_node, json_object_ref (json_object));
+
+	return document;
+}
+
 /**
  * couchdb_document_get:
  * @couchdb: A #CouchdbSession object
@@ -102,7 +117,7 @@ couchdb_document_new (CouchdbSession *couchdb)
  * which case, the error argument will contain information about the error.
  */
 CouchdbDocument *
-couchdb_document_get (CouchdbSession *couchdb,
+couchdb_document_get (CouchdbSession *session,
 		      const char *dbname,
 		      const char *docid,
 		      GError **error)
@@ -111,16 +126,16 @@ couchdb_document_get (CouchdbSession *couchdb,
 	JsonParser *parser;
 	CouchdbDocument *document = NULL;
 
-	g_return_val_if_fail (COUCHDB_IS_SESSION (couchdb), NULL);
+	g_return_val_if_fail (COUCHDB_IS_SESSION (session), NULL);
 	g_return_val_if_fail (dbname != NULL, NULL);
 	g_return_val_if_fail (docid != NULL, NULL);
 
 	encoded_docid = soup_uri_encode (docid, NULL);
-	url = g_strdup_printf ("%s/%s/%s", couchdb_session_get_uri (couchdb), dbname, encoded_docid);
+	url = g_strdup_printf ("%s/%s/%s", couchdb_session_get_uri (session), dbname, encoded_docid);
 	parser = json_parser_new ();
-	if (couchdb_session_send_message (couchdb, SOUP_METHOD_GET, url, NULL, parser, error)) {
+	if (couchdb_session_send_message (session, SOUP_METHOD_GET, url, NULL, parser, error)) {
 		document = g_object_new (COUCHDB_TYPE_DOCUMENT, NULL);
-		document->couchdb = couchdb;
+		document->session = session;
 		document->dbname = g_strdup (dbname);
 
 		document->root_node = json_node_copy (json_parser_get_root (parser));		
@@ -159,7 +174,7 @@ couchdb_document_put (CouchdbDocument *document,
 	const char *id;
 	JsonParser *parser;
 	gboolean result = FALSE;
-	gboolean send_ok;;
+	gboolean send_ok;
 
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), FALSE);
 	g_return_val_if_fail (dbname != NULL, FALSE);
@@ -171,13 +186,13 @@ couchdb_document_put (CouchdbDocument *document,
 		char *encoded_docid;
 
 		encoded_docid = soup_uri_encode (id, NULL);
-		url = g_strdup_printf ("%s/%s/%s", couchdb_session_get_uri (document->couchdb), dbname, encoded_docid);		
-		send_ok = couchdb_session_send_message (document->couchdb, SOUP_METHOD_PUT, url, body, parser, error);
+		url = g_strdup_printf ("%s/%s/%s", couchdb_session_get_uri (document->session), dbname, encoded_docid);		
+		send_ok = couchdb_session_send_message (document->session, SOUP_METHOD_PUT, url, body, parser, error);
 
 		g_free (encoded_docid);
 	} else {
-		url = g_strdup_printf ("%s/%s/", couchdb_session_get_uri (document->couchdb), dbname);
-		send_ok = couchdb_session_send_message (document->couchdb, SOUP_METHOD_POST, url, body, parser, error);
+		url = g_strdup_printf ("%s/%s/", couchdb_session_get_uri (document->session), dbname);
+		send_ok = couchdb_session_send_message (document->session, SOUP_METHOD_POST, url, body, parser, error);
 	}
 
 	if (send_ok) {
@@ -193,9 +208,9 @@ couchdb_document_put (CouchdbDocument *document,
 		}
 
 		if (id)
-			g_signal_emit_by_name (document->couchdb, "document_updated", dbname, document);
+			g_signal_emit_by_name (document->session, "document_updated", dbname, document);
 		else
-			g_signal_emit_by_name (document->couchdb, "document_created", dbname, document);
+			g_signal_emit_by_name (document->session, "document_created", dbname, document);
 
 		result = TRUE;
 	}
@@ -241,12 +256,12 @@ couchdb_document_delete (CouchdbDocument *document, GError **error)
 	if (!id || !revision) /* we can't remove a document without an ID and/or a REVISION */
 		return FALSE;
 
-	url = g_strdup_printf ("%s/%s/%s?rev=%s", couchdb_session_get_uri (document->couchdb), document->dbname, id, revision);
+	url = g_strdup_printf ("%s/%s/%s?rev=%s", couchdb_session_get_uri (document->session), document->dbname, id, revision);
 	
 	/* We don't parse the http response, therefore the parser arg is NULL */
-	if (couchdb_session_send_message (document->couchdb, SOUP_METHOD_DELETE, url, NULL, NULL, error)) {
+	if (couchdb_session_send_message (document->session, SOUP_METHOD_DELETE, url, NULL, NULL, error)) {
 		result = TRUE;		
-		g_signal_emit_by_name (document->couchdb, "document_deleted", document->dbname, id);
+		g_signal_emit_by_name (document->session, "document_deleted", document->dbname, id);
 	}
 
 	g_free (url);
